import { EditorView } from '@codemirror/view';
import { Compartment, EditorState } from '@codemirror/state';
import { sql, MySQL } from '@codemirror/lang-sql';
import { basicSetup } from '@tidbcloud/codemirror-extension-basic-setup';
import { getSqlStatements, getNearbyStatement, sqlParser } from '@tidbcloud/codemirror-extension-sql-parser';
import { getCurStatements, curSql } from '@tidbcloud/codemirror-extension-cur-sql';

const langSql = (config) => sql({
    dialect: MySQL,
    upperCaseKeywords: true,
    ...config
});
class SQLEditorInstance {
    constructor(editorId, editorView, themeCompartment, sqlCompartment, extraData) {
        Object.defineProperty(this, "editorId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: editorId
        });
        Object.defineProperty(this, "editorView", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: editorView
        });
        Object.defineProperty(this, "themeCompartment", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: themeCompartment
        });
        Object.defineProperty(this, "sqlCompartment", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: sqlCompartment
        });
        Object.defineProperty(this, "extraData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: extraData
        });
    }
    changeTheme(theme) {
        if (this.themeCompartment.get(this.editorView.state) === theme)
            return;
        this.editorView.dispatch({
            effects: this.themeCompartment.reconfigure(theme)
        });
    }
    changeSQLConfig(sqlConfig) {
        this.editorView.dispatch({
            effects: this.sqlCompartment.reconfigure(langSql(sqlConfig))
        });
    }
    getAllStatements() {
        return getSqlStatements(this.editorView.state);
    }
    getCurStatements() {
        return getCurStatements(this.editorView.state);
    }
    getNearbyStatement() {
        const { from } = this.getCurStatements()[0];
        return getNearbyStatement(this.editorView.state, from);
    }
}
const createSQLEditorInstance = ({ editorId, doc, basicSetupOptions = {}, sqlConfig = {}, theme = [], extraExts = [], extraData = {} }) => {
    const themeCompartment = new Compartment();
    const sqlCompartment = new Compartment();
    const extensions = [
        // make it full height default
        // you can override it by theme and extraExts
        EditorView.theme({
            '&': { height: '100%' },
            '.cm-line': {
                paddingLeft: '8px'
            }
        }),
        basicSetup({
            foldGutter: false,
            foldKeymap: false,
            searchKeymap: true,
            autocompletion: false,
            ...basicSetupOptions
        }),
        themeCompartment.of(theme),
        sqlCompartment.of(langSql(sqlConfig)),
        sqlParser(),
        curSql(),
        extraExts
    ];
    const editorView = new EditorView({
        state: EditorState.create({
            doc,
            extensions
        })
    });
    const editorInst = new SQLEditorInstance(editorId, editorView, themeCompartment, sqlCompartment, extraData);
    return editorInst;
};

//-------------
class EditorCache {
    constructor() {
        Object.defineProperty(this, "cache", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "addEditor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (editorId, editor) => {
                this.cache.set(editorId, editor);
            }
        });
        Object.defineProperty(this, "getEditor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (editorId) => {
                return this.cache.get(editorId);
            }
        });
        Object.defineProperty(this, "deleteEditor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (editorId) => {
                this.cache.delete(editorId);
            }
        });
        Object.defineProperty(this, "clearEditors", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: () => {
                this.cache.clear();
            }
        });
    }
}

export { EditorCache, SQLEditorInstance, createSQLEditorInstance };
